/*!	\file
	\brief MAX7456 Driver
*/
/* Driver and some functions for MAX7456
	It does not contain code specific to an application

	Created by JR 101026

	MRU JR 110823
	110823 Removed loopcntr from MAX7456_init()
	110414 Changed MAXdisplay_lines to exit immediately if zero line length
		Included delay.h and replaced delay_ms()
	110109 Coding of @ in translation table corrected
	101231 Added MAXreadregs
	101230 Changed init code
	101229 Corrected MAX7456dsptext(), increment text pntr
		Correct
	101203 Changed loop counter in MAX7456read_byte()
	101202 Changed font translation table to start at ASCII 000
	101201 Wrote MAX7456_init()

Can distinguish odd and even frames by looking at CV level at start
of VSYNC ISR
*/
#include "config.h"
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <inttypes.h>
#include <util/delay.h>
#include "MAX7456.h"
#include "SIMRCV.h"
#include "SPI.h"

#if VTIDEVICE == MAX7456OSD

/********************
	local RAM variables with initialisation as needed
*********************/
#ifdef DEBUGMAX7456
uint8_t maxregs[REGCNT];
#endif
/********************
	local PROM variables with initialisation
*********************/
/*! Character code, this is used when there is no English char font
	available for the ASCII code
*/
#define NOSM 0x50
/*!
	\brief Character translation table for factory set font.

	THis is used because the character ROM addresses do not correspond to
	the ASCII codes.
	Order as ASCII codes to character address in MAX7456.
	Here only the same characters as in the SW_OSD space to Z.
	Index into the table with MAXdef_font[ASCII]
	Source, MAX7456 datasheet, http://pdfserv.maxim-ic.com/en/ds/MAX7456.pdf
*/

const uint8_t MAXdef_font[] PROGMEM =
{	NOSM, /*   000  */
	NOSM, /*   001  */
	NOSM, /*   002  */
	NOSM, /*   003  */
	NOSM, /*   004  */
	NOSM, /*   005  */
	NOSM, /*   006  */
	NOSM, /*   007  */
	NOSM, /*   010  */
	NOSM, /*   011  */
	NOSM, /*   012  */
	NOSM, /*   013  */
	NOSM, /*   014  */
	NOSM, /*   015  */
	NOSM, /*   016  */
	NOSM, /*   017  */
	NOSM, /*   020  */
	NOSM, /*   021  */
	NOSM, /*   022  */
	NOSM, /*   023  */
	NOSM, /*   024  */
	NOSM, /*   025  */
	NOSM, /*   026  */
	NOSM, /*   027  */
	NOSM, /*   030  */
	NOSM, /*   031  */
	NOSM, /*   032  */
	NOSM, /*   033  */
	NOSM, /*   034  */
	NOSM, /*   035  */
	NOSM, /*   036  */
	NOSM, /*   037  */
	0x00, /*   040  */
	NOSM, /* ! 041  */
	0x48, /* " 042  */
	NOSM, /* # 043  */
	NOSM, /* $ 044  */
	NOSM, /* % 045  */
	NOSM, /* & 046  */
	0x46, /* ' 047  */
	0x3F, /* ( 050  */
	0x40, /* ) 051  */
	NOSM, /* * 052  */
	NOSM, /* + 053  */
	0x45, /* , 054  */
	0x49, /* - 055  */
	0x41, /* . 056  */
	0x47, /* / 057  */
	0x0A, /* 0 060  */
	0x01, /* 1 061  */
	0x02, /* 2 062  */
	0x03, /* 3 063  */
	0x04, /* 4 064  */
	0x05, /* 5 065  */
	0x06, /* 6 066  */
	0x07, /* 7 067  */
	0x08, /* 8 070  */
	0x09, /* 9 071  */
	0x44, /* : 072  */
	0x43, /* ; 073  */
	0x4A, /* < 074  */
	NOSM, /* = 075  */
	0x4B, /* > 076  */
	0x42, /* ? 077  */
	0x4C, /* @ 100  */
	0x0B, /* A 100  */
	0x0C, /* B 101  */
	0x0D, /* C 102  */
	0x0E, /* D 103  */
	0x0F, /* E 104  */
	0x10, /* F 105  */
	0x11, /* G 106  */
	0x12, /* H 107  */
	0x13, /* I 110  */
	0x14, /* J 111  */
	0x15, /* K 112  */
	0x16, /* L 113  */
	0x17, /* M 114  */
	0x18, /* N 115  */
	0x19, /* O 116  */
	0x1A, /* P 117  */
	0x1B, /* Q 120  */
	0x1C, /* R 121  */
	0x1D, /* S 122  */
	0x1E, /* T 123  */
	0x1F, /* U 124  */
	0x20, /* V 125  */
	0x21, /* W 126  */
	0x22, /* X 127  */
	0x23, /* Y 130  */
	0x24  /* Z 131  */
};
/********************
	local EEPROM variables with initialisation
*********************/

/********************
	external variable definitions
	RAM, PROM, EEPROM
*********************/
/* SIM or RCV */
/*! Lines for VTI */
extern char
 	timeline[],
 	headline[],
 	info1line[],
	info2line[],
	spareline[];

/* RS232 */
/* SWOSD or MAX7456 or neither */
/* DOGM optional */
/* GPSRCV or GPSSIM */
/* SHUTTER optional */
/* dispatcher, almost always */


/********************
	external function definitions
*********************/
extern uint8_t SPIread(void);
extern uint8_t SPIwrite(uint8_t data);
/********************
	Functions
 ********************/
/*!
	\brief Read a byte from the MAX7456
	\param[in] regaddr Register address
	\return Register value

	MAX7456read_byte() reads the specified register. It assumes that the
	device is not busy. It does not test the register address
	No bit bashing here,
	see http://www.roboternetz.de/phpBB2/viewtopic.php?t=46674
*/
uint8_t MAX7456read_byte(uint8_t regaddr)
{	uint8_t regval;

	INIT_MAXSPI									/* Set SPCR, SPSR */
	SELMAX7456
	SPIwrite(regaddr);					/* Write the register address */
	regval = SPIwrite(regaddr);	/* This clock the value back into SPDR */
	DESELMAX7456

	return regval;
} /* MAX7456read_byte */

/*!
	\brief Read a character address and attributes from the MAX7456
	\param[in] regaddr Register address
	\return Character in MSB and attributes in LSB

	Sets to 16-bit mode, return value is:
*/

unsigned int MAX7456rdCA_CATR(uint8_t regaddr)
{
	INIT_MAXSPI									/* Set SPCR, SPSR */
	SELMAX7456									/* Select MAX7456 */
/* not yet implemented */
	DESELMAX7456								/* Deselect MAX7456 */
	return 0;
} /* MAX7456rdCA_CATR */

/*!
	\brief Write a byte to the MAX7456
	\param[in] regaddr Register address
	\param[in] val Value to write in register
*/
void MAX7456write_byte(uint8_t regaddr, uint8_t val)
{	INIT_MAXSPI									/* Set SPCR, SPSR */
	SELMAX7456									/* Select MAX7456 */
	SPIwrite(regaddr);					/* write the register address */
	SPIwrite(val);							/* Write the value */
	DESELMAX7456								/* Deselect MAX7456 */
} /* MAX7456write_byte */

/*!
	\brief initialise the MAX7456

	At start up time the video source may not be connected.\n
	Enable device, and in the field end ISR look at LOS, if it's
	1 there is no video in. Display"CONNECT VIDEO IN" in the headline,
	changing the standard every 50 fields (use the field counter).
	If it's 0, initialise dsplines according to the standard, this in
	HW_OSD
	Change output signal to detected input signal
	It takes at least 50ms to complete, it waits for power on reset
*/
void MAX7456_init(void)
{ uint8_t regval;

/* Wait for reset finished, this assumes that the reset pin is at
	logical 1, because we can't read STAT while it's low
*/
	DESELMAX7456
	/* Wait until power on reset is finished, In the HW the RESET pin is at
	+5V, so we can read the STAT register. During a reset pulse one
	cannot read the registers
*/
	while ((MAX7456read_byte(STATR) & STATRSTMSK) == (uint8_t) RESETBUSY)
		;
/* Initialise device, NTSC, internal sync, display enable,
	at next VSYNC, no SW reset, video buffer enable
*/
	MAX7456write_byte(VM0W, SWRESET);
	_delay_ms(1);							/* wait for reset to finish, can't read */
/* Set automatic black level control on */
	MAX7456setbits(OSDBL, BLCON, OSDBLBLCMSK);
/* Set display on immediately, internal sync, NTSC. If it's set to
	autosync, but the detected signal is different from what is set, the
	VSYNC signal disappears, and hence the interrupt
*/
	MAX7456write_byte(VM0W, DSPON | SSMAUTO);
	_delay_ms(100);						/* wait for sync */
	if (((regval = MAX7456read_byte(STATR)) & (PALON | NTSCON)) != 0)
	{	regval = ((regval & STATPALMSK) == PALON) ? VSSPAL : VSSNTSC;
/* sets video standard, external sync, immediate VSYNC,
	video buffer enable
	Try leaving in autosync mode
*/
//	MAX7456write_byte(VM0W, regval | DSPON | SSMEXT);
		MAX7456write_byte(VM0W, regval | DSPON);
	}

/* Initialise the video standard in the FIELD_ENDhndlr() if no
	video in connected here
*/
} /* MAX7456_init */

/*!
	\brief MAX7456_ClearDisplay() clears the display memory, then waits
	until it's finished
*/
void MAX7456_ClearDisplay(void)
{ uint8_t DMMreg = MAX7456read_byte(DMMR);

	MAX7456write_byte(DMMW, DMMreg | (uint8_t) CLDON);
/* Wait until the clear display memory has finished, flag is cleared
	automatically
*/
	while ((MAX7456read_byte(DMMR) & DMMCLDMASK) == CLDON)
		;
} /* MAX7456_ClearDisplay */

/*!
	\brief 16-bit write text to display memory.
	\param textp Null terminated string, max length CHARSPERLN
	\param transtab	Pointer to ASCII to MAX7456 character code table in PROM
	\param attributes Local background, blink, invert
	\param line Screen line number 0-12 (NTSC), 0-15 (PAL), not tested
	\param column Column number, 0 CHARSPERLN - 1 (29)
	Exits immediately if *textp == '\0'
*/
void MAX7456dsptext(
	char 		*textp, 				/* null terminated string, max CHARSPERLN */
	PGM_P 	transtab,				/* ASCII to MAX7456 character code */
	int8_t 	attributes, 		/* attributes, local background, blink, inv */
	int8_t 	line,						/* Line number 0-12 (NTSC), 0-15 (PAL) */
	int8_t	column)					/* column number, 0 CHARSPERLN - 1 (29) */
{ unsigned int memaddr;		/* Current char memory address */

	if (*textp == '\0')
		return;
	memaddr = line * CHARSPERLN + column;
/* write DMAH bit 8 */
	MAX7456setbits(DMAH, (uint8_t) (memaddr >> 8), DMABIT8MSK);
/* Set CMA low byte */
	MAX7456write_byte(DMALW, (uint8_t) (memaddr & 0x00ff));
/* set 16 bit mode, autoincrement and attribute bytes in DMM */
	MAX7456setbits(DMM, (OPM16 | (attributes & DMMATTRMSK) | AIMENBL),
		(DMMOPMMASK | DMMATTRMSK | DMMAIMMASK));
/* write characters */
	for ( ; *textp != '\0'; textp++)
	{	SELMAX7456
		SPIwrite(pgm_read_byte((PGM_P) &MAXdef_font[(uint8_t) *textp]));
		DESELMAX7456
	} /* for */
	SELMAX7456
	SPIwrite(AIMEND);						/* End auto-increment mode */
	DESELMAX7456
} /* MAX7456dsptext */

/*!
	\brief MAX7456readCM reads the character memory into an array
	\param	CA Character address
	\param	CMpixels Pointer to character font array. See p. 38 of the datasheet for the method
*/
void MAX7456readCM(uint8_t CA,	/* character address */
	CMpixels *pixels)							/* Array[CMBYTES] of uint8_t */
{	uint8_t *pixelp = (uint8_t *) pixels;

	MAX7456setbits(VM0, DSPOFF, VM0DSPMSK);	/* display off */
	MAX7456write_byte(CMAHW, CA);				/* character address */
	MAX7456write_byte(CMMW, READNVM);		/* NVM to shadow memory */
	while (MAX7456read_byte(CMMR) != 0)	/* wait for completion */
		;
	for (uint8_t i = 0; i < CMBYTES; i++, pixelp++)
	{	MAX7456write_byte(CMALW, i);				/* select byte */
	 	*pixelp = MAX7456read_byte(CMDOR);	/* read byte to array */
	}
	MAX7456setbits(VM0, DSPON, VM0DSPMSK);		/* display on */
} /* MAX7456readCM */

/*!
	\brief Write VM0, VM1, DMM, DMAH, DMAL, CMM, CMAH, CMAL, OSDBLR and STAT
 		to regs[]. Call in 16 bit mode, not checked.
	 \param regs Pointer to array[10] for register values
	 Is used for diagnostics.
*/
void MAXreadregs(uint8_t *regs)
{
	*regs++ = MAX7456read_byte(VM0R);
	*regs++ = MAX7456read_byte(VM1R);
	*regs++ = MAX7456read_byte(DMMR);
	*regs++ = MAX7456read_byte(DMAHR);
	*regs++ = MAX7456read_byte(DMALR);
	*regs++ = MAX7456read_byte(CMMR);
	*regs++ = MAX7456read_byte(CMAHR);
	*regs++ = MAX7456read_byte(CMALR);
	*regs++ = MAX7456read_byte(OSDBLR);
	*regs   = MAX7456read_byte(STATR);

} /* MAXreadregs */

/********************
	Interrupt service routines
*********************/

/********************
	Event handlers
*********************/


#endif /* MAX7456OSD */
