/*!	\file
	\brief Header for MAX7456 driver
*/
/*
	Created 110110
	MRU 110920
	110920 Moved HW definitions from SIMRCV.h
	110704 Added INIT_MAXSPI
	110110 Added character memory definitions
	110105 Added DIFFNDX
	110101 Corrected auto increment mode enable bit
	101229 Added macro MAX7456setbits(REG, BITS, MASK)
	MRU 101201
	101201 Added STAT definitions

*/
#ifndef _MAX7456_H_
	#define _MAX7456_H_

#include <avr/io.h>
#include <avr/pgmspace.h>
#include "myportbits.h"

/* Only one of JR_DEFS, SVEN_DEFS and XMEGA_DEFS are defined */
#if (HARDWARE == SVEN_DEFS)
/* No MAX7456 in Sven's HW */
#endif /* SVEN_DEFS */

#if (HARDWARE == JR_DEFS) && (VTIDEVICE == MAX7456OSD)
/* Port/pin definitions */
/* Composite video INT0/PD2 negative flank. We don't need the
	interrupt for MAX7456, but we do need to read HSYNC in the
	VSYNC ISR to see if it's an even or odd field
*/
	#define CVPORT 		PORTD
	#define CVDDR 		DDRD
	#define CVPINR 		PIND
	#define CVPORTPIN PORTD2
	#define CVDD 			DDD2
	#define CVPIN 		PIND2

/* Vertical sync PB0, interrupt */
	#define VSYNC_vect	PCINT0_vect
	#define VSYNCPORT 	PORTB
	#define VSYNCDDR 		DDRB
	#define VSYNCPINR 	PINB
	#define VSYNCPORTPIN PORTB0
	#define VSYNCDD 		DDB0
	#define VSYNCPIN 		PINB0
	#define VSYNCIBIT 	PCINT0
	#define VSYNCMSKREG PCMSK0		/* Pin change mask register */

	#define INIT_MUXS0 					/* No multiplexer for MAX7456 */
	#define INIT_OE 						/* No odd/even pin for MAX7456 */

/* MAX7456 definitions, this is for the prototype board, CS pin
	I/O is with the SPI pins
*/
	#define MAXCSPORT	PORTB
	#define MAXCSDDR 	DDRB
	#define MAXCSPINR PINB
	#define MAXCSPIN 	PB2
/* INITMAX7456 does not set up the SPI, see INIT_MAXSPI below */
	#define INITMAX7456 \
		sbi(MAXCSDDR, MAXCSPIN);	/* CS as output */	\
		DESELMAX7456							/* deselect MAX7456 */
/* No CV interrupt */
	#define ENBL_CVINT
	#define DSBL_CVINT

/* Initialise composite video, interrupt not needed  */
	#if (VTIDEVICE == MAX7456OSD)
		#define INIT_CV \
		cbi(CVDDR, CVPIN);				/* Set as input */
	#endif

	#define VSYNCINT_ON 					/* VSYNC interrupt on */ \
		setbit(VSYNCMSKREG, VSYNCIBIT);	/* Allow VSYNC pin int */		\
		setbit(PCICR, PCIE0);				/* enable PCINT 0 to 7 */

	#define VSYNCINT_OFF 					/* Disable VSYNC interrupt */ \
		clrbit(VSYNCMSKREG, VSYNCIBIT);

/* Macro to initialise VSYNC HW */
	#define INIT_VSYNC \
		cbi(VSYNCDDR, VSYNCPIN);			/* Set as input */					\
		VSYNCINT_ON										/* Enable interrupt */

#endif /* JR_DEFS*/

#if (HARDWARE == XMEGA_DEFS)
/* Port/pin definitions */
	#if (PGMFUNCTION == RECEIVER)

	#endif /* RECEIVER */

	#if (PGMFUNCTION == SIMULATOR)

	#endif /* SIMULATOR */
#endif /* XMEGA_DEFS*/
/* Lines in display */
#define NTSC_LINES	13
#define PAL_LINES		16
#define CHARSPERLN	30		/* Characters in a line */
/* size of array for MAXreadregs() */
#define REGCNT 10

/*! Write register addresses, name as in datasheet followed by W */
enum WRREGADDR
{	VM0W,						/* Video mode 0 */
	VM1W,						/* Video mode 1 */
	HOSW,						/* Horizontal offset */
	VOSW,						/* Vertical offset */
	DMMW,						/* Display memory mode */
	DMAHW,					/* Display Memory Address High */
	DMALW,					/* Display Memory Address Low */
	DMDIW,					/* Display Memory Data In */
	CMMW,						/* Character Memory Mode */
	CMAHW,					/* Character Memory Address High */
	CMALW,					/* Character Memory Address Low */
	CMDIW,					/* Character Memory Data In */
	OSDMW,					/* OSD Insertion Mux */
	RB0W,						/* Row 0 Brightness */
	RB1W,						/* Row 1 Brightness */
	RB2W,						/* Row 2 Brightness */
	RB3W,						/* Row 3 Brightness */
	RB4W,						/* Row 4 Brightness */
	RB5W,						/* Row 5 Brightness */
	RB6W,						/* Row 6 Brightness */
	RB7W,						/* Row 7 Brightness */
	RB8W,						/* Row 8 Brightness */
	RB9W,						/* Row 9 Brightness */
	RB10W,					/* Row 10 Brightness */
	RB11W,					/* Row 11 Brightness */
	RB12W,					/* Row 12 Brightness */
	RB13W,					/* Row 13 Brightness */
	RB14W,					/* Row 14 Brightness */
	RB15W,					/* Row 15 Brightness */
	OSDBLW = 0x6C		/* OSD Black Level */
};
/*! Read register addresses, name as in datasheet followed by R
	Read address always has MSB set
*/
#define READFLG 0x80
enum RDREGADDR
{	VM0R = READFLG,	/* Video mode 0 */
	VM1R,						/* Video mode 1 */
	HOSR,						/* Horizontal offset */
	VOSR,						/* Vertical offset */
	DMMR,						/* Display memory mode */
	DMAHR,					/* Display Memory Address High */
	DMALR,					/* Display Memory Address Low */
	DMDIR,					/* Display Memory Data In */
	CMMR,						/* Character Memory Mode */
	CMAHR,					/* Character Memory Address High */
	CMALR,					/* Character Memory Address Low */
	CMDIR,					/* Character Memory Data In */
	OSDMR,					/* OSD Insertion Mux */
	RB0R,						/* Row 0 Brightness */
	RB1R,						/* Row 1 Brightness */
	RB2R,						/* Row 2 Brightness */
	RB3R,						/* Row 3 Brightness */
	RB4R,						/* Row 4 Brightness */
	RB5R,						/* Row 5 Brightness */
	RB6R,						/* Row 6 Brightness */
	RB7R,						/* Row 7 Brightness */
	RB8R,						/* Row 8 Brightness */
	RB9R,						/* Row 9 Brightness */
	RB10R,					/* Row 10 Brightness */
	RB11R,					/* Row 11 Brightness */
	RB12R,					/* Row 12 Brightness */
	RB13R,					/* Row 13 Brightness */
	RB14R,					/* Row 14 Brightness */
	RB15R,					/* Row 15 Brightness, 0x9F */
	OSDBLR = 0xEC,	/* OSD Black Level */
	STATR  = 0xA0, 	/* Status, read only */
	DMDOR	 = 0xB0, 	/* Display Memory Data Out, read only */
	CMDOR	 = 0xC0 	/* Character Memory Data Out, read only */
};
/* All arguments must be uint8_t */
#define SETVAR(var, mask, val) \
	(((uint8_t) var & ~(uint8_t) mask) | ((uint8_t) val & (uint8_t) mask))
/* Register definitions from datasheet */
/*! VM0 bits, Video mode 0, Address 0x00/0x80 */
#define VM0VSSMSK 	((uint8_t) 0x40)
#define VSSNTSC 		((uint8_t) 0x00)	/* NTSC */
#define VSSPAL	 		((uint8_t) 0x40)	/* PAL */
#define VM0SSMMSK 	((uint8_t) 0x30)
#define SSMAUTO			((uint8_t) 0x00)	/* Select sync mode auto */
#define SSMEXT			((uint8_t) 0x20)	/* Select sync mode external */
#define SSMINT			((uint8_t) 0x30)	/* Select sync mode internal */
#define VM0DSPMSK 	((uint8_t) 0x08)
#define DSPOFF			((uint8_t) 0x00)	/* OSD display off */
#define DSPON				((uint8_t) 0x08)	/* OSD display on */
#define VM0VSMSK		((uint8_t) 0x04)
#define VSIMM				((uint8_t) 0x00)	/* VSYNC immediate */
#define VSNXT				((uint8_t) 0x04)	/* VSYNC on next VSYNC input */
#define SWRESET 		((uint8_t) 0x02)	/* Software reset */
#define VM0VBNMSK		((uint8_t) 0x01)
#define DSPENB			((uint8_t) 0x00)	/* Display enable */
#define DSPDIS			((uint8_t) 0x01)	/* Display disable */

/*! VM1 bits, video mode 1, Address 0x01/0x81 */
#define VM1BCKMSK		((uint8_t) 0x80)	/* Background mode */
#define BCKLOCAL		((uint8_t) 0x00)	/* default set by DMM[5], DMD[7] */
#define BCKGREY			((uint8_t) 0x80)	/* Grey background */
#define VM1BKMMSK 	((uint8_t) 0x70)	/* Background bright mode */
#define BKM00				((uint8_t) 0x00)	/* 0% */
#define BKM07				((uint8_t) 0x10)	/* 7% */
#define BKM14				((uint8_t) 0x02)	/* 14% */
#define BKM21				((uint8_t) 0x30)	/* 21% */
#define BKM28				((uint8_t) 0x40)	/* 28%  default */
#define BKM35				((uint8_t) 0x50)	/* 35% */
#define BKM42				((uint8_t) 0x60)	/* 42% */
#define BKM49				((uint8_t) 0x70)	/* 49% */
#define VM1BLKMSK 	((uint8_t) 0x0C)	/* Blink time mask, BT */
#define BLINK2			((uint8_t) 0x00)	/* 2 fields */
#define BLINK4			((uint8_t) 0x04)	/* default 4 fields */
#define BLINK6			((uint8_t) 0x08)	/* 6 fields */
#define BLINK8			((uint8_t) 0x0C)	/* 8 fields */
#define VM1DTYMSK		((uint8_t) 0x03)	/* Blink duty cycle */
#define DUTY11			((uint8_t) 0x00)	/* Duty cycle 1:1 BT */
#define DUTY12			((uint8_t) 0x01)	/* Duty cycle 1:2 BT */
#define DUTY13			((uint8_t) 0x02)	/* Duty cycle 1:3 BT */
#define DUTY31			((uint8_t) 0x03)	/* default Duty cycle 3:1 BT */

/*! HOS register bits. Horizontal offset register HOS, Address 0x02/0x82 */
#define HOSMASK			((uint8_t) 0x3F)	/* Bits 7,6 not used */
#define HOSLEFT			((uint8_t) 0x00)	/* 32 pixels to left */
#define HOSMID			((uint8_t) 0x20)	/* default 0 pixels offset */
#define HOSRIGHT		((uint8_t) 0x3F)	/* 31 pixels to right */

/*! VOS Vertical offset register, Address 0x03/0x83 */
#define VOSMASK			((uint8_t) 0x1F)	/* Bits 7,6 not used */
#define VOSLEFT			((uint8_t) 0x00)	/* 16 pixels up */
#define VOSMID			((uint8_t) 0x10)	/* default 0 pixels offset */
#define VOSRIGHT		((uint8_t) 0x1F)	/* 15 pixels down */

/*! DMM register,  Display memory mode, Address 0x04/0x84*/
#define DMMOPMMASK	((uint8_t) 0x40)	/* Op mode mask */
#define OPM16				((uint8_t) 0x00)	/* 16 bit mode, default */
#define OPM8				((uint8_t) 0x40)	/* 8 bit mode */
#define DMMLBCMASK	((uint8_t) 0x20)	/* local background control, 16 bit */
#define LBCVIN			((uint8_t) 0x00)	/* Video IN if external sync, def */
#define LBCBKM			((uint8_t) 0x20)	/* set to background */
#define DMMBLKMASK	((uint8_t) 0x10)	/* Blink control, 16 bit */
#define BLKOFF			((uint8_t) 0x00)	/* blink off, default */
#define BLKON				((uint8_t) 0x10)	/* blink on */
#define DMMINVMASK	((uint8_t) 0x08)	/* Invert pixel mask, 16 bit mode */
#define DMMATTRMSK  (DMMLBCMASK | DMMBLKMASK | DMMINVMASK) /* Attributes */
#define INVOFF			((uint8_t) 0x00)	/* chars are white, default */
#define INVON				((uint8_t) 0x08)	/* chars are black */
#define DMMCLDMASK	((uint8_t) 0x04)	/* Clear display memory */
#define CLDOFF			((uint8_t) 0x00)	/* Normal mode, don't clear, def */
#define	CLDON				((uint8_t) 0x04)	/* Clear display memory */
#define DMMVSCMASK	((uint8_t) 0x02)	/* Vertical sync clear */
#define VSCIMDT			((uint8_t) 0x00)	/* def, clear dsp memory if CLDON */
#define VSCNEXT			((uint8_t) 0x02)	/* clear dsp memory on next VSYNC */
#define DMMAIMMASK	((uint8_t) 0x01)	/* Auto-increment mode */
#define AIMDSBL			((uint8_t) 0x00)	/* def. Auto increment disable */
#define AIMENBL			((uint8_t) 0x01)	/* Auto increment enable */
#define AIMEND			((uint8_t) 0xFF)	/* end auto increment write */

/*! DMAH register,  Display memory address high, Address 0x05/0x85 */
#define DMAHBSBMASK	((uint8_t) 0x02)	/* Byte selection bit */
#define BSBCHAR			((uint8_t) 0x00)	/* DMAH[1], DMAL is character addr */
#define BSBATTR			((uint8_t) 0x02)	/* DMAH[1], DMAL is attribute addr */
#define DMABIT8MSK	((uint8_t) 0x01)	/* Character memory addr bit 8 */

/*! CMAH register, Character memory address, address 0x0A/0x8A */
#define CMALBYTMASK	((uint8_t) 0x3F		/* Byte address for four pixels */

/* Character attributes */

/*! STAT register, read only, address 0xa0 */
#define STATRSTMSK 	((uint8_t) 0x40)
#define RESETBUSY		((uint8_t) 0x40)		/* not finished reset */
#define RESETRDY		((uint8_t) 0x00)		/* Reset finished */
#define STATCMSMSK	((uint8_t) 0x20)		/* Character memory status */
#define CMSAVAIL		((uint8_t) 0x00)		/* CMS available */
#define CMSUNAVAIL	((uint8_t) 0x20)		/* CMS unavailable */
/*! VSYNC, HSYNC later as needed */
#define STATLOSMSK	((uint8_t) 0x04)		/* Loss of sync */
#define LOSINSYNC		((uint8_t) 0x00)		/* Sync'sd with video in */
#define LOSNOSYNC		((uint8_t) 0x04)		/* Not sync'd with video in */
#define STATNTSCMSK	((uint8_t) 0x02)		/* NTSC bit */
#define NTSCOFF			((uint8_t) 0x00)		/* No NTSC detected */
#define NTSCON			((uint8_t) 0x02)		/* NTSC detected */
#define STATPALMSK	((uint8_t) 0x01)		/* PAL bit */
#define PALOFF			((uint8_t) 0x00)		/* No PAL detected */
#define PALON				((uint8_t) 0x01)		/* PAL detected */

/*! OSDBL register, address 6C/EC */
#define OSDBLBLCMSK	((uint8_t) 0x10)		/* Automatic black level */
#define BLCON				((uint8_t) 0x00)		/* Black level control on */
#define BLCOFF			((uint8_t) 0x10)		/* Black level control off */
/* Do not change bits 0-3, they are factory set */

/*! MAX7456 SPI setup is in MAX7456.h */
#define SELMAX7456		cbi(MAXCSPORT, MAXCSPIN);
#define DESELMAX7456 	sbi(MAXCSPORT, MAXCSPIN);

/* VSYNC_FLANK value is POS_FLANK if pin VSYNCPIN of port VSYNCPORT has value 1,
otherwise NEG_FLANK
*/
#define VSYNC_FLANK PC_FLANK(VSYNCPINR, VSYNCPIN)

/* Line numbers in display, these can change for NTSC. The text
	lines in the display must be spaced by at least FONTLINES scanlines
	PAL has 576 visible lines, NTSC has 486 (odd + even field)
	Fix the headline, info1 and info2 lines as adjacent lines at the top,
	Move the time line to the last line
	For SW OSD the line number is where the top line appears
	Avoid lines near thetop for the time being, need some time to build
	the texts after the VSYNC
*/
/*! Line numbers for PAL */
enum PALLINENOS {PALHEADLN = 1, PALINFO1LN = 13,
	PALINFO2LN = 14, PALTIMELN = 15, PALSPARELN = 2};
/*! Line numbers for NTSC */
enum NTSCLINENOS {NTSCHEADLN = 1, NTSCINFO1LN = 10,
	NTSCINFO2LN = 11, NTSCTIMELN = 12, NTSCSPARELN = 2};
/* DIFFNDX is the index into dsplines for which the line number
	is different in NTSC and PAL, used in HW_OSD.c
*/
#define DIFFNDX TIMNDX
/* 16 lines for PAL, 13 for NTSC */

/* Some definitions for reading and writing character memory
	Number of bytes for pixels in one character */
#define CMBYTES 54
/* array for storing pixel data */
typedef uint8_t CMpixels[CMBYTES];

/*! Pixel types */
#define BLACKPIXEL 0x00
#define WHITEPIXEL 0x02
#define TRNGRAYPIXEL	0x01		/* Transparent (ext sync), grey (int) */
/* Positions of the pixels in byte, left shift count */
#define LEFTPIXEL 6						/* left pixel */
#define LMIDPIXEL 4						/* left middle */
#define RMIDPIXEL	2						/* right middle */
#define RGHTPIXEL 0						/* right pixel */
#define PIXELMASK 0x03				/* mask for pixel value */
/* Commands for NVM read/write */
#define READNVM		0x50				/* read NVM into shadow memory */
#define WRITENVM	0xA0				/* write shadow memory to NVM */

/* Macros */
/* INIT_MAXSPI sets up the SPCR and SPSR registers for the MAX7456
	These may be different from those of the DOGM display, so it must
	be executed at every read and write, and NOT be interrupted
*/
#define INIT_MAXSPI \
/* Enable SPI, master, CPOL = CPHA = 0, MSB first, Fosc/4 */	\
	SPCR = (1 << SPE) | (1 << MSTR);	\
	SPSR = 0;										/* Single speed */

/*!
	\brief Set bits in the MAX7456 registers
	\param[in] REG is the register name
	\param[in] BITS are the bits to be changed, they are ANDed with MASK
	\param[in] MASK is the bitmask, bit value 1 where a register bit is to be changed
*/
#define MAX7456setbits(REG, BITS, MASK)	\
	MAX7456write_byte(REG ## W, 						\
	(MAX7456read_byte(REG ## R) & ~MASK) | (BITS & MASK))

/* Function definitions */

uint8_t MAX7456read_byte(uint8_t regaddr);
unsigned int MAX7456rdCA_CATR(uint8_t regaddr);
void MAX7456write_byte(uint8_t val, uint8_t regaddr);
void MAX7456_init(void);
void MAX7456_ClearDisplay();
void MAX7456dsptext(
	char *text, 							/* null terminated string, max CHARSPERLN */
	PGM_P		transtab,					/* ASCII to MAX7456 CA translation table */
	int8_t	attributes, 			/* attributes, 0 if none */
	int8_t 	line,							/* Line number 0-12 (NTSC), 0-15 (PAL) */
	int8_t	column);					/* column number, 0 CHARSPERLN - 1 (29) */
void MAXreadregs(uint8_t *regs);
void MAX7456readCM(uint8_t CA, CMpixels *pixels);
#endif /* _MAX7456_H_ */
