/*! \file
\brief Some general defines and macros

*/
/*
	Header for simulator and receiver program versions
	TODO Move data definitions and enums to appropriate header files

	MRU JR 110905
	110919 Moved HW definitions to the appropriate header files
	110905 Minor changes
	110705 Added enable pullup in INIT_BUTTONS, Sven HW
	110619 Added enable pullup to LW_INIT
	110613 Added enable pullup in INIT_BUTTONS, JR HW
	110604	Changed DOGM CS to D7, pin C6 was dead
	110418 Added UNDEFTX to enum LWSIG
	110316 Added DOGM power on/off macros
	110303 Corrected DOGM initialisation macro
	110203 Changed SPI initialisation for SW_OSD
	110122 Tidied up file, no changes
	110119 Added some macros for LW and GPS sync status
	110118 Added GPS_ALARM, changed PPS_ABSENT to GPS_ABSENT
	110115 Changed definition for RXPOLL
	110114 Reversed red LED signal output in JR_DEFS
	110113 Added FASTTICK, SLOWTICK and TIMERFREQ
	110103 Changed SPI clock frequency to Fosc/4
	101220 Changed INIT_SPI
	101211 Moved initialisation back to SIMRCV.h, easier to manage
	101206 Moved all initialisation to the appropriate modules
		Execution depends on the modules configured in config.h
	101123 Added definitions for EEflags
	101119 Added JR_DEFS for HW
	101117 Changed TICKSPERSEC to 1000 if MAX7456 (receiver or sim)
	101116 Changed DSPLINES struct to include start char position
	101109 Added definitions for JR HW
	101105 First created

 ***** Bugs *****

 ***** TTDL *****

	*************************
	defines, typedefs and include files
	*************************

*/
#ifndef _SIMRCV_H_
#define _SIMRCV_H_

#define SWMAJREV 1
#define SWMNRREV 1
#define SWBLDNO  21

#include "myportbits.h"
#include <inttypes.h>
#include "config.h"
#include "errorno.h"

/* Length of the site name in EEPROM */
#define SITENAMELN 16

/* *******************************
	defines, typedefs and structs follow, they are
	grouped according to the various modules
*********************************/

/* **************** TIMER1 ****************/
/* *************** Controls ***************/
/* ************** SIM or RCV **************/
/* Define a string in flash, do this to save RAM */
#define PGMSTRING(STRNAME, STR) const char PROGMEM pgm ## STRNAME[] = STR;
#define EXTPGMSTRING(STRNAME) extern const char pgm ## STRNAME[];

#if (VTIDEVICE == MAX7456OSD)
	#define MAXSCREENTXTLN 30
#endif /* MAX7456OSD */
/* Only time for about 26 characters with SW OSD */
#if (VTIDEVICE == SW_OSD)
	#define MAXSCREENTXTLN 26
#endif /* SW_OSD */
/* **************** RS232 *****************/
/* ****** SWOSD or MAX7456 or neither *****/
/* Letter in time line for odd/even field, this is set in the field
	end ISR, variable oddoreven
*/
#define ODDLETTER 	'O'
#define EVENLETTER 	'E'

/* Definitions for the field counter. To save time in coversion to the
	display format (string) it's in BCD. Six digits are enough for
	20000 sec in PAL, or 16666 2/3 sec in NTSC
*/
#define BCDDIGITCNT 6
#define BCDBYTES ((BCDDIGITCNT + 1) / 2)
typedef uint8_t BCD[BCDBYTES];

/* Video signal format, distinguish by line count in field in SW_OSD,
	or by STAT register bits 1,0 in  MAX7456
*/
enum VFORMAT {UNDEFVIDEO, NTSC, PAL};

/* For font table in SWOSD.c */
#define FONT_LINES 7
#define FIRST_CHAR ' '
#define LAST_CHAR 'Z'
#define FONT_CHARS (LAST_CHAR - FIRST_CHAR + 1)

typedef uint8_t FONT_ARRAY[FONT_CHARS][FONT_LINES];

#define MAXDSPLINES 5		/* Maximum number of lines to be displayed */
/* Indices in the DISPLAYLINE array, see below */
enum dspndx {HEADNDX, INFO1NDX, INFO2NDX, TIMENDX, SPARENDX};

/*! \struct displayline
\brief struct for display lines.

scanline: for SW_OSD this is the scan line, or for MAX7456 the line no, this
	depends on whether it's NTSC (13 lines) or PAL (16 lines). In SW_OSD it is
	set in the FIELD_ENDhndlr()

*/
typedef struct displayline
{
/* for SW_OSD this is the scan line, or for MAX7456 the line no, this
	depends on whether it's NTSC (13 lines) or PAL (16 lines)
*/
	int			scanline;						/*!< Line where the text is inserted. See note above */
	uint8_t	attributes;					/*!< only used for MAX7456 */
	int8_t	startpos;						/*!< position in characters, not used in SW_OSD */
	uint8_t	textlng;						/*!< character count in line */
	char 		*linetext;					/*!< points to null terminated string */
} DISPLAYLINE;

/* ************ DOGM optional *************/
/* *********** GPSRCV or GPSSIM ***********/

#if (defined GPSSIM) || (defined GPSRCV)
	#if (GPSMODEL == GARMINLVC18x)
/* Maximum number of characters in a GPS 18x sentence + 1 for NUL */
		#define BFRLNG 83
/* maximum field length in GPRMC */
		#define GPRMCMXFLD 10
/* Latitude is ddmm.dddd = 9 char */
		#define GPSLATLEN 9
/* Longitude is dddmm.dddd = 10 char */
		#define GPSLONGLEN 10
/* For GPGGA */
		#define GPGGAMXFLD 10
		#define GPGGAHDOPLN 4
		#define GPGGAANHGTLN 7
		#define GPGGAGHGT 6
		#define GPGGANSATLN 2		/* number of satellites, 2 characters */

/* Length of PPS pulse is a multiple (n + 1) * PPSUNIT ms 0 <= n <= 48
	for the GPS 18x, max n is different for GPS18x 5, see manual
	It is changed by field 13 of the PGMRC sentence
*/
		#define PPSUNIT 20
/* Default length of the PPS pulse in ms, min is 1 unit */
		#define PPSLENGTH ((4 + 1) * PPSUNIT)
	#endif /* GARMINLVC18x */
#endif /* GPSSIM || GPSRCV */

/* Maximum length of a GPS sentence name */
#define MAXNAMELN 6

/* For setting the offset between PPS and timer 1 in prescaler ticks. This
	should prevent jitter of the 1ms counter in tm_msec if the quarz
	frequency isn't exact. Used in the PPS interrupt. It ensures that the
	PPS interrupt always comes after the TIMER1 interrupt, simplifies the code
*/
#define GPSOFFSET 4

/* Tolerances for the difference between the internal clock and the PPS
	INTPPSTOL is for drift between INTtime and PPS arrival time, we don't correct
	the internal clock if the error is <= INTPPSTOL prescaler ticks
	PPSARVTOL	is for error in the arrival time of the PPS relative to PPS. A large error
	may be caused by spurious PPS pulses. IN this case we'll drop back to the internal
	clock and resync on the next PPS
*/
#define INTPPSTOL 1 /* Predivider ticks, F_CPU / PREDIV */
#define PPSARVTOL 5

#define FIXDELAY	10	/* Time to first fix in sec for simulator < 60sec  */
/* Number of equal GPStime and INTtime comparisons until the
	INTtime is defined as synchronised with GPStime
*/
#define GPSSYNCCOUNT 10
/* flags for error_flags variable to control the output */
#define POSLEAPSECERR 	((uint8_t) (1 << 0))		/* Transmit a positive leap second */
#define NEGLEAPSECERR 	((uint8_t) (1 << 1))		/* Transmit a negative leap second */
#define DELAYFIX				((uint8_t) (1 << 2))		/* Send GPRMC with status V for FIXDELAY sec */
#define PARITYERR				((uint8_t) (1 << 3))		/* Parity error in GPRMC */
#define PPSARVERR				((uint8_t) (1 << 4))		/* make error in PPS arrival time */
#define PPSNSRTERR			((uint8_t) (1 << 5))		/* insert a PPS pulse */
#define RMTCROSSSEC			((uint8_t) (1 << 6))		/* GPRMC starts in one sec, finishes next */

/* default latitude and longitude, only needed  for SIMULATOR */
#if (PGMFUNCTION == SIMULATOR)
	#define DEFLATITUDE "5006.9910"
	#define DEFLATHEMIS 'N'
	#define DEFLONGITUDE "0829.9120"
	#define DEFLONGHEMIS 'E'
	/* For GPGGA */
	#define DEFHDOP "12.1"			/* For GPGGA, HDOP */
	#define DEFANTHGT "146.0"		/* For GPGGA, antenna height M */
	#define DEFGDHGT "0050.0"		/* For GPGGA, geoid height, M */
#else
	#define DEFLATITUDE ""
	#define DEFLATHEMIS 'N'
	#define DEFLONGITUDE ""
	#define DEFLONGHEMIS 'E'
	/* For GPGGA */
	#define DEFHDOP   ""				/* For GPGGA, HDOP */
	#define DEFANTHGT ""				/* For GPGGA, antenna height M */
	#define DEFGDHGT  ""				/* For GPGGA, geoid height, M */
#endif
/* For GPGSA */

/* For testing errors in PPS interval in SIMULATOR, not used yet */
#define PPSSHORTINT (TICKSPERSEC - PPSARVTOL - 1)
#define PPSLONGINT (TICKSPERSEC + PPSARVTOL + 1)

/* ******* LW_SIM or LW_RCV or none *******/

/* Bits for EEflags and RAMflags */
#define LWNEGSIG 		((uint8_t) 0x00)	/* Signal is negative pulse */
#define LWPOSSIG 		((uint8_t) 0x01)	/* Signal is positive pulse */
#define LWPOLMSK		((uint8_t) 0x01)	/* LW polarity mask */

/* Names of LW signals, used in evaluating signal, this is in LWsignal
	in EEPROM. LWTRNCNT is a counter for the number of transmitters
	If you change the values here you must change the PROM tables in
	LW.c correspondingly. Must be consecutive values for -SIG up to LWTRNCNT
*/
enum LWSIG{UNDEFTX = -1, DCF77SIG = 0, MSFSIG, WWVBSIG, JJYSIG, LWTRNCNT};

/* Tolerances for LW (DCF77, WWVB etc.) time signals. Change these after
	measurements. We measure the arrival time error of the negative LW
	flank over a minute, and correct at the end of the minute. LWcalcError()
	delivers the error in prescalar ticks
*/
#define INTLWTOL 	5
#define LWARVTOL	(1 * PREDVPERMS)	/* digits = number of msec */

/* For testing errors in LW interval in LW_SIM, not used yet */
#define LWSHORTINT (TICKSPERSEC - LWARVTOL - 1)
#define LWLONGINT (TICKSPERSEC + LWARVTOL + 1)

/* Error codes when checking LW data, used for example is DCFtest() */
#define DCFA1ERR 		1			/* DCF bit A1, flag ZNCCHGA error */
#define DCFZ1Z2ERR 	2			/* Flag STFLAG error */
#define DCFLPSERR 	3			/* Leap second announcement error */
#define DCFUNXPERR 	4			/* unexpected error */
#define TMCMPERR		5			/* tmcmp() returned error in DCFtest */
#define DCFCHECKERR	6			/* DCFcheck() returned error */

/* Definitions for LW operations. The execution is specific to a
	particular transmitter, If you change the values of LWINIT etc you
	must change the PROM tables in LW.c correspondingly
*/
enum LWOPS
	{	LWINIT, 						/* initialise the LW simulator/receiver */
		LWWASMARK, 					/* Was the mark at the previous second? receiver */
		LWCHECK,						/* Check signal integrity in receiver */
		LWSIG2TIME, 				/* convert signal to struct GPSTM in receiver */
		LWTIME2SIG					/* convert struct GPSTM in simulator */
	};
/* For field counter, use BCD to make it faster and shorter, 3 bytes are
	enough for 20000 seconds PAL, 16666 2/3 sec NTSC
*/

/* *********** SHUTTER optional ***********/

/* Definitions for timer functions */

typedef struct timer
{	uint16_t	tmr_ms;			/*!< delay time in ticks, decremented in tick ISR */
	uint16_t	tmr_param;	/*!< parameter for event */
	uint8_t		tmr_event;	/*!< event/priority to fire off when delay reaches 0 */
} TIMER_STRUCT;

/* Number of timers */
#define TIMER_COUNT 3

/* Flags for timing events and errors, max 16, these are used in reporting errors */
enum timerflags
{	tmrLEAPMS	= 0x0001,		/* correction to millisecond counter */
	tmrLEAPPS	= 0x0002,		/* correction to prescaler tick */
	tmrNOPPS	= 0x0004,		/* PPS missing when expected */
	tmrPPSARV	= 0x0008,		/* Unexpected PPS arrival time */
	tmrRMCPAR = 0x0010,		/* GPRMC parity error */
	tmrINTRMC = 0x0020,		/* Internal time and GPRMC time/date not equal */
	tmrLWSYNC = 0x0040,		/* LW clock is now in sync */
	tmrLWSNCERR = 0x0080,	/* LW signal and clock comparison error */
	tmrRMCXSEC 	= 0x0100	/* GPRMC started in one sec, end in next */
};

/* ********** dispatcher,always ***********/
/* ************ string_util.c *************/


/******************************************
	Some general definitions, used in more than one module
******************************************/

/* Definitions used after pin change interrupt */
#define POS_FLANK 1
#define NEG_FLANK 0
/* IN PC_FLANK() REG must be the pin register, not the port */
#define PC_FLANK(REG, BITNO) \
	(((REG & (1 << BITNO)) != (uint8_t) 0) ? POS_FLANK : NEG_FLANK)


/* The following definitions are hardware dependent for devices with one
	SPI channel
	Port and bit definitions for all modules are here to make it easier
	to avoid conflicting definitions
	Macro definitions are in the appropriate module .h file
*/
/* Pin definitions for SPI, this is when I'm master SPI */
#define DDR_SPI 	DDRB			/* Data direction register for SPI */
#define PINR_SPI 	PINB			/* Pin register for SPI */
#define SPI_SS		PB2				/* Chip select, user control when master */
#define SPI_MOSI 	PB3				/* Set to output, data to slave */
#define SPI_MISO 	PB4				/* enable SPI makes this an input */
#define SPI_SCK 	PB5				/* Clock output */
/* INIT_SPIPINS sets MOSI, SCK and SS pins as output
	The clock phase (CPOL/CPHA) and speed are set differently for MAX7456 and
	SW_OSD, MAX7456 and DOGM, see below
*/
#define INIT_SPIPINS DDRB |= ((1 << SPI_MOSI) | (1 << SPI_SCK) | (1 << SPI_SS));

#if (defined SW_OSD) || (defined MAX7456OSD)
	#if (VTIDEVICE == SW_OSD)
		#define SPISPEED (1 << SPI2X)		/* double speed */
	#else
		#define SPISPEED 0							/* single speed */
	#endif
#endif /* (defined SW_OSD) || (defined MAX7456OSD) */

/* Need different setup for DOGM */
#ifndef DOGMDISPLAY
	#define INIT_DOGMSPI
#endif /* DOGMDISPLAY */

/*! \enum daysofweek
	Days of week, counted as in ISO
*/
enum daysofweek {MONDAY = 1, TUESDAY, WEDNESDAY, THURSDAY, FRIDAY, SATURDAY,
	SUNDAY};
/* Function definitions */
/*
int8_t isleapyear(GPSTM *gpstime);
void ticksec(GPSTM *gpstime);
void clearBCD(BCD *bcdp);
void incBCD(BCD *bcdp);
void initGPS(void);
*/
#endif /* _SIMRCV_H_ */
