/*! \file
\brief Configuration file config.h for GPSsim program, it defines the modules used

	The values are verified in config_rules.h, and generate compile errors
	if a rule is violated
*/

/*
	Created 101103 by JR

	MRU 111006
	111006 Added Doxygen comments
	101122 Added file config_rules.h to test the config
	101121 Removed restriction that SIMULATOR can only transmit
		sentences and RECEIVER can only receive sentences

*/
#ifndef _CONFIG_H
	#define _CONFIG_H
#include "options.h"

/*! The following part defines which modules are used for generating
	the program.\n\n
	Set PGMFUNCTION to SIMULATOR or RECEIVER
*/
#define	PGMFUNCTION RECEIVER

/*! Hardware definitions, set to SVEN_DEFS, JR_DEFS or XMEGA_DEFS */
#define HARDWARE JR_DEFS
/*! Use a GPS receiver as time source, or simulate a GPS receiver */
#define		GPS
/*! Use an LW receiver as time source, or simulate an LW transmitter */
#define 	LWTIME
/*!
	DOGMDISPLAY is on the board
	The type is at the moment fixed as a DOGM162, 2 lines of 16 characters
	The interface is also fixed at SPI
*/
#define	DOGMDISPLAY			/* DOGM display */
/* If DOGMDISPLAY is defined, specify the following symbols */
#ifdef DOGMDISPLAY			/* Only DOGM162 at the moment */
	#define DOGMTYPE DOGM162	/* one of DOGM161, DOGM162, DOGM163 DOGM81 */
/*! \def DOGMINTERFACE
	The interface is DOGM8BIT, DOGM4BIT or DOGMSPI
*/
	#define DOGMINTERFACE DOGMSPI
/*!	\def DOGMSUPPLY
	Supply voltage is DGM5V or DOGM33V
*/
	#define DOGMSUPPLY DOGM5V
#endif /* DOGMDISPLAY */

/*! Control buttons, see file controls.h */
#define CONTROLS

/*!
	VTI device, NO_OSD, SW_OSD or MAX7456OSD
*/
#define VTIDEVICE MAX7456OSD
/*!
	Canon shutter control, not yet implemented
*/
#undef CANON_TIMER			/* Canon shutter control */
/*!
	GPS device type.
	At the moment only Garmin LVC 18x (GARMINLVC18x) is implemented.
	The baud rate is defined in RS232.h, later it will use autobaud
*/
#define GPSMODEL GARMINLVC18x		/* GPS device type */

#if (PGMFUNCTION == RECEIVER)
	#ifdef GPS
		#define GPSRCV
		#define RX_RS232
	#endif /* GPS */
	#ifdef LWTIME
/*!
	In the RECEIVER version LWDEFSIGNAL can be one of the defined transmitters
	(see enum LWSIG in SIMRCV.h), or defined as UNDEFTX. The program will then recognise
	the transmitter automatically.
*/
		#define LWDEFSIGNAL UNDEFTX		/* auto detect transmitter */
		#define LW_RCV			/*!< Receive LW signal */
/* define one or more transmitters */
		#define	DCF77				/*!< receive DCF signal */
		#undef MSF					/*!< receive MSF signal */
		#undef	WWVB				/*!< Not yet implemented */
		#undef	JJY					/*!< Not yet implemented */
	#endif /* LWTIME */
/*! If the receiver should also transmit NMEA sentences, then
	define TX_RS232
*/
	#undef TX_RS232
#endif /* RECEIVER */

#if (PGMFUNCTION == SIMULATOR)
	#ifdef GPS
		#define GPSSIM
		#define TX_RS232
	#endif /* GPS */
	#ifdef LWTIME
/*!
	LWDEFSIGNAL must be set to one of the defined transmitters
*/
		#define LWDEFSIGNAL DCF77	/*!< default transmitter, D */
		#define	LW_SIM			/* Simulate LW signal */
/*! define one or more of DCF77, MSF, WWVB and JJY */
		#define	DCF77				/*!< simulate DCF signal */
		#undef	MSF					/*!< simulate MSF */
		#undef	WWVB				/*!< Not yet implemented */
		#undef	JJY					/*!< Not yet implemented */
	#endif /* LWTIME */
/* If the simulator should also receive sentences, then define RX_RS232 */
	#undef RX_RS232
#endif /* SIMULATOR */

/*! F_CPU, CPU quarz frequency in Hz */
#define F_CPU 18432000
/*! \brief PREDIV, Timer predivider setting, see the processor documentation

	This determines the internal timing source interrupt frequency.
	In the SIMULATOR version F_CPU / PREDIV must be a whole multiple of 1000
	In the RECEIVER version F_CPU / PREDIV must be a whole multiple of 10
*/
#define PREDIV 1024

/*! \brief DEFSITENAME Sitename, max SITENAMELN characters.

	The character set is limited, for the SW_OSD version see the variable font in
	file SW_OSD.c. For the MAX7456OSD version see the MAX7456 data sheet
*/
/*									 1234567890123456 */
#define DEFSITENAME "A37 MUEGGELHEIM "

/* Defines for debugging code */
#undef DEBUGOSD
/* For recording leading LW flank */
#undef	DEBUGLWPULSE
/* Define none or one of DEBUGLWDRIFT or DEBUGPPSDRIFT */
#undef	DEBUGLWDRIFT
#undef DEBUGPPSDRIFT
#undef	DEBUGMAX7456

/* config_rules.h checks the configuration */
#include "config_rules.h"

#endif	/* _CONFIG_H */
